/* $Id: metadata.c,v 1.1.1.1 2005/05/28 17:24:27 bedlam Exp $ */
/* metadata.c
 * Copyright (c) 2001-3 Brendan Cully <brendan@xiph.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include "definitions.h"

#define INITDELAY 500000

extern stream404_config_t stream404_config;

static char* Artist = NULL;
static char* Title = NULL;
static char* Filename = NULL;

/* define conversion table to print unpritable ascii chars */
#define A 97
#define AAC 226 /*  */
#define AU 227  /*  */
#define AAE 225 /*  */
#define AO 228 /*  */
#define AAG 224 /*  */
#define AO2 229 /*   */

#define AMAJ 65 
#define AMAJU 195 /*  */
#define AMAJO 197 /*  */
#define AMAJACCG 192 /*  */
#define AMAJACCE 193 /*  */
#define AMAJACCCIRC 194 /*  */
#define AMAJTREM 196 /*  */

#define C 99
#define CC 231 /*   */

#define CMAJ 67
#define CMAJC 199 /*   */

#define E 101 
#define EAE 233 /*   */
#define EAC 234 /*   */
#define EU 235 /*   */
#define EAG 232 /*   */

#define EMAJ 69
#define EMAJAG 200 /*  */
#define EMAJAE 201 /*  */
#define EMAJACC 202 /*  */
#define EMAJU 203 /*  */

#define I 105
#define ITREM 239 /*  */
#define IACC 238 /*  */
#define IACCG 236 /*  */
#define IACCE 237 /*  */

#define O 111
#define OCIRC 244 /*  */
#define OTREM 246 /*  */
#define OU 245 /*  */
#define OACE 243 /*   */
#define OACG 242 /*  */

#define OMAJ 79
#define OMAJTREM 213 /*  */
#define OMAJACCG 210 /*  */
#define OMAJACCE 211 /*  */
#define OMAJACCC 212 /*  */
#define OMAJU 214 /*  */

#define N 110
#define NTREM 241 /*  */

#define NMAJ 78
#define NMAJTREM 209 /*   */

#define U 117
#define UTREM 252 /*   */
#define UACCCIRC 251 /*  */
#define UACCG 249 /*  */
#define UACCE 250 /*  */

#define UMAJ 85
#define UMAJTREM 220 /*  */
#define UMAJACCG 217 /*  */
#define UMAJACCE 218 /*  */
#define UMAJACCC 219 /*  */

#define Y 121
#define YTREM 255 /*  */

#define YMAJ 89
#define YMAJACCE 221 /*  */

#define DEFAULT 32 /* Space */

#define ASCIIUPPERLIMIT 127

/* Private function declarations */
static char* metadata_clean_filename (char* path, char* buf,
				      size_t len);
static void metadata_update (int delay, stream404_stream_t *stream);

char *clean_char(char *string)
{
  int i = 0;
  int c;
  int len = strlen(string);
  
  if (string == NULL) 
    return NULL;

  for (i = 0; i < len; i++) {
    c = *(string + i);
    if (c < 0) {
      c += 256;
     /* compare to a char */
      if ((c ==  AAC) || (c == AU) || (c == AAE) || (c == AO) || (c == AAG) ||
	  (c == AO2)) {
	*(string + i) = A;
	continue;
      }
      /* compare to A char */
      if ((c == AMAJU) || (c == AMAJO) || (c == AMAJACCG) || (c == AMAJACCE)
	  || (c == AMAJACCCIRC) || (c ==AMAJTREM)) {
	*(string + i) = AMAJ;
	continue;
      }

      /* compare to c char */
      if (c == CC) {
	*(string + i) = C;
	continue;
      }
      /* compare to C char */
      if (c == CMAJC) {
	*(string + i) = CMAJC;
	continue;
      }
      
      /* compare to e char */
      if ((c == EAE) || (c == EAC) || (c == EU) || (c == EAG)) {
	*(string + i) = E;
	continue;
      }
      /* compare to E char */
      if ((c == EMAJAG) || (c == EMAJAE) || (c == EMAJACC) || (c == EMAJU)) {
	*(string + i) = EMAJ;
	continue;
      }

      /* compare to i char */
      if ((c == ITREM) || (c == IACC) || (c == IACCG) || (c == IACCE)) {
	*(string + i) = I;
	continue;
      }

      /* compare to o char */
      if ((c == OCIRC) || (c == OTREM) || (c == OU) || 
	  (c == OACE) || (c == OACG)) {
	*(string + i) = O;
	continue;
      }
      /* compare to O char */
      if ((c == OMAJTREM) || (c == OMAJACCG) || (c == OMAJACCE) || 
	  (c == OMAJACCC) || (c == OMAJU)) {
	*(string + i) = OMAJ;
	continue;
      }

      /* compare to n char */
      if (c == NTREM) {
	*(string + i) = N;
	continue;
      }
      /* compare to N char */
      if (c == NMAJTREM) {
	fprintf(stderr, "N\n");
	*(string + i) = NMAJ;
	continue;
      }

      /* compare to u char */
      if ((c == UTREM) || (c == UACCCIRC) || (c == UACCG) || (c == UACCE)) {
	*(string + i) = U; 
	continue;
      }
      /* compare to U char */
      if ((c == UMAJTREM) || (c == UMAJACCG) || (c == UMAJACCE) ||
	  (c == UMAJACCC)) {
	*(string + i) = UMAJ;
	continue;
      }

      if (c == YTREM) {
	*(string + i) = Y;
	continue;
      }

      if (c == YMAJACCE) {
	*(string + i) = YMAJ;
	continue;
      }

      /* default */
      *(string + i) = DEFAULT;
    }
  }
  return string;
}


/* Global function definitions */

void
stream404_metadata_get (char* artist, size_t alen, char* title, size_t tlen)
{
  if (Artist)
    snprintf (clean_char(artist), alen, "%s", Artist);
  if (Title)
    snprintf (clean_char(title), tlen, "%s", Title);
}

void
stream404_metadata_set (char* artist, char* title)
{
  stream404_util_free (Artist);
  Artist = NULL;
  stream404_util_free (Title);
  Title = NULL;

  if (artist && *artist)
    Artist = stream404_util_strdup (clean_char(artist));
  if (title && *title)
    Title = stream404_util_strdup (clean_char(title));
}

void
stream404_metadata_get_file (char* filename, size_t len)
{
    if (Filename)
	snprintf (clean_char(filename), len, "%s", Filename);
}

void
stream404_metadata_set_file (char* filename)
{
  char buf[1024];

  stream404_util_free (Filename);
  Filename = NULL;

  if (filename && *filename) {
    metadata_clean_filename (filename, buf, sizeof (buf));
    Filename = stream404_util_strdup (clean_char(buf));
  }
}

/* Update metadata on server via fork.
 * Note that the very first metadata update after connection is delayed,
 * because if we try to update our new info to the server and the server has
 * not yet accepted us as a source, the information is lost. */
/* launch as many child than needed */
void
stream404_metadata_update (int delay)
{
  stream404_stream_t* stream;
  pid_t child;

  for (stream = stream404_config.streams; stream; stream = stream->next) {
   
    /* never metaupdate with an ogg vorbis file */
    if (stream->type == STREAM404_OUTPUT_MP3) {
      if (delay)
	stream404_log_debug ("Delaying metadata update...");

      if ((child = fork()) == 0) {
	setproctitle("stream404", "metadata delayed");
	metadata_update (delay ? INITDELAY : 0, stream);
	_exit (0);
      }
      
      if (child == -1)
	stream404_log_debug ("Metadata update failed: fork");
    }
  }
}

static void
metadata_update (int delay, stream404_stream_t *stream)
{
  shout_metadata_t* metadata;
  char song[1024];
  char* value = NULL;
  int rc;

  if (delay)
    usleep (delay);

 
  if (Title) {
    if (Artist)
      snprintf (song, sizeof (song), "%s - %s", Artist, Title);
    else
      snprintf (song, sizeof (song), "%s", Title);
  } else
    snprintf (song, sizeof (song), "%s", Filename);
  
  value = song;

  if (value == NULL) {
    return;
  }  

  if (!(metadata = shout_metadata_new())) {
    stream404_log_error ("Error allocating metadata structure");
    return;
  }

  if (shout_metadata_add (metadata, "song", value) != SHOUTERR_SUCCESS) {
    stream404_log_error ("Error adding info to metadata structure");
    shout_metadata_free (metadata);
    return;
  }

  if (stream->type == STREAM404_OUTPUT_MP3) {
    rc = shout_set_metadata (stream->conn, metadata);
    
    if (rc != SHOUTERR_SUCCESS)
      stream404_log_error ("Updating metadata on %s failed.", stream->mount);
    else
      stream404_log_debug ("Updated metadata on %s to: %s", stream->mount, value);
  } else if (stream->type == STREAM404_OUTPUT_VORBIS) {
    /* never metaupdate with an ogg vorbis file */
  }
  
  shout_metadata_free (metadata);
}


/* Cleanup a filename so it looks more like a song name */
static char *
metadata_clean_filename (char* path, char *buf, size_t len)
{
  char *ptr =NULL;

  if (!path || !buf) {
    stream404_log ("ERROR: Polluted strings sent to filename cleanup.");
    return NULL;
  }

  /* Find last slash */
  ptr = strrchr (path, '/');

  if (ptr && strlen (ptr) > 0) {
    strncpy (buf, ptr + 1, len);
  } else {
    strncpy (buf, path, len);
  }

  if ((ptr = strrchr (buf, '.'))) {
    *ptr = '\0';
  }

  stream404_log_debug ("Filename cleaned up from [%s] to [%s]", path, buf);
  return buf;
}
