/* $Id: sql_builtin.c,v 1.3 2005/06/24 21:18:12 bedlam Exp $ */
/* sql_playlist.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 *
 *
 *
 *
 SQL Table Format 

 

 */

#include <definitions.h>
#include <mysql/mysql.h>
#include <signal.h>
#include <pthread.h>
#include "rand.h"


static MYSQL mysql; 
static MYSQL_RES *res; 
static MYSQL_ROW row;

static FILE* fp = NULL;
/*
static time_t playlist_modtime = 0;
*/
static int lineno = 0;

extern stream404_config_t stream404_config;
static playlist_module_t *sqlpm;

/* Private function declarations */
static char* playlist_sql_builtin_get_next (void *);
static char* playlist_sql_builtin_get_next_urg (void *);
static int playlist_sql_builtin_get_lineno (void);
static void playlist_sql_builtin_shutdown (void);
static char* playlist_sql_get_metadata(void);

static int playlist_sql_builtin_open_playlist (playlist_module_t* pm);
/* kept for future purpose
static int playlist_sql_builtin_reopen_playlist (playlist_module_t* pm);
*/

void launch_loop(void);
/*
static void playlist_sql_builtin_shuffle_playlist (void);
*/
/*
static int playlist_sql_builtin_line_skip (int lineno, FILE* fp);
*/

/* Global function definitions */

/* Initialize the builting playlist handler */
int
stream404_playlist_sql_initialize (playlist_module_t* pm)
{
  /* used in get_next() */
  sqlpm = pm;
  
  stream404_log_debug ("Initializing builting SQL playlist handler...");

  pm->get_next = playlist_sql_builtin_get_next;
  pm->get_next_urg = playlist_sql_builtin_get_next_urg;
  pm->get_metadata = playlist_sql_get_metadata;
  pm->get_lineno = playlist_sql_builtin_get_lineno;
  pm->shutdown = playlist_sql_builtin_shutdown;

  if (!playlist_sql_builtin_open_playlist (pm)) {
    stream404_log ("Could not connect to sql server.");
    stream404_setup_shutdown ();
    return -1;
  }

  /* as this module is a sql one we need according to the conditions
     lookup as much as we need to the urgent table */
  launch_loop();

  lineno = 0;
  return 1;
}

static char *
playlist_sql_builtin_get_next (void *arg)
{
  char *sql_text = "SELECT `S_ID`, `S_Code`, `S_Argument1`, `S_Argument2`, `S_Argument3`, `S_Argument4`, `S_Argument5`, `S_Argument6` FROM %s order by S_ID LIMIT 1";
  char *sql_text_del = "DELETE FROM %s WHERE S_ID = %s ";
  //char *sql_currentstreamtable = "t_currentstream";
  char *sql_text_del_current = "DELETE FROM %s";
  char *sql_text_current = "INSERT into %s SELECT `S_Argument4`, now(), `S_UserID`, `S_Argument8`, `S_TrackID`, `S_DataID`, `S_Argument7`, `S_Argument1` FROM %s where S_ID = %s";
  //char *sql_streamlogtable = "t_streamlog";
  char *sql_text_stream_log = "INSERT into %s (S_Type,D_Datetime,S_DataID,S_FileName,S_Description,S_PlayTime) SELECT `S_Argument4`, now(), `S_DataID`, `S_Argument1`, `S_Argument6`, `S_Argument7` FROM %s where S_ID = %s";

  char c[4096];
  char *out, *cur;
  input_stream_t *source = arg;

  /* set these values to 0 */
  source->type = 0;
  source->spec = 0;

  /*
  if (!mysql) {
    * *hem* sql connect has been droped *
    if (!playlist_sql_builtin_open_playlist (sqlpm)) {
      stream404_log ("Could not connect to sql server.");
      stream404_setup_shutdown ();
      return NULL;
    }
  }
  */

  snprintf(c, strlen(sql_text) + strlen(stream404_config.sql.table), sql_text, stream404_config.sql.table); 

  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    return NULL;
  }
  
  res = mysql_store_result(&mysql);
  
  if (mysql_num_rows(res) == 0) {
    /* there is no next file to be played !! */
    stream404_log ("OMG ! No next file to play !!");
    mysql_free_result(res);
    return NULL;
  } 
  
  row = mysql_fetch_row(res);
  
  
  /* purge current stream table*/
  snprintf(c, strlen(sql_text_del_current) + strlen(stream404_config.sql.currentstream_table), sql_text_del_current, stream404_config.sql.currentstream_table);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    mysql_free_result(res);
    return NULL;
  }
  /* then update current stream table with the current song played */
  snprintf(c, strlen(sql_text_current) + strlen(stream404_config.sql.currentstream_table) + strlen(stream404_config.sql.table) + strlen(row[0]), sql_text_current, stream404_config.sql.currentstream_table, stream404_config.sql.table, row[0]);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    mysql_free_result(res);
    return NULL;
  }

  /* log current stream with the current song played */
  snprintf(c, strlen(sql_text_stream_log) + strlen(stream404_config.sql.streamlog_table) + strlen(stream404_config.sql.table) + strlen(row[0]), sql_text_stream_log, stream404_config.sql.streamlog_table, stream404_config.sql.table, row[0]);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error log stream sql query: %s", mysql_error(&mysql));
    mysql_free_result(res);
    return NULL;
  }


  
  /* delete this record */
  snprintf(c, strlen(sql_text_del) + strlen(stream404_config.sql.table) + strlen(row[0]), sql_text_del, stream404_config.sql.table, row[0]);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    mysql_free_result(res);
    return NULL;
  }

  /* check for the second argument */
  cur = row[1];
  
  if (cur && (strncmp(cur, "PLAY", 4) == 0)) {
    source->spec = _FILE;
    /* get the file location */
    cur = row[2];
    if (cur && (strncasecmp(cur, "http://", 7) == 0)) {
      source->location = REMOTE;
    } else {
      source->location = LOCAL;
    }
    cur = row[3];
    if (cur && (strncasecmp(cur, "MP3", 3) == 0)) {
      source->type = STREAM404_INPUT_MP3;
    } 
    if (cur && (strncasecmp(cur, "OGG", 3) == 0)) {
      source->type = STREAM404_INPUT_VORBIS;
    } 
    if (cur && (strncasecmp(cur, "MP4", 3) == 0)) {
      source->type = STREAM404_INPUT_MP4;
    }
    if (cur && (strncasecmp(cur, "FLAC", 4) == 0)) {
      source->type = STREAM404_INPUT_MP4;
    }
    else {
      /* should stay as STREAM404_INPUT_UNKNOWN */
    }
  } else if (cur && (strncmp(cur, "LIVE", 4) == 0)) {
    source->location = REMOTE;
    source->spec = STREAM;
    cur = row[3];
    if (cur && (strncasecmp(cur, "MP3", 3) == 0)) {
      source->type = STREAM404_INPUT_MP3;
    } 
    if (cur && (strncasecmp(cur, "OGG", 3) == 0)) {
      source->type = STREAM404_INPUT_VORBIS;
    } 
    if (cur && (strncasecmp(cur, "MP4", 3) == 0)) {
      source->type = STREAM404_INPUT_MP4;
    }
    if (cur && (strncasecmp(cur, "FLAC", 4) == 0)) {
      source->type = STREAM404_INPUT_MP4;
    }
    else {
      /* should stay as STREAM404_INPUT_UNKNOWN */
    }
    cur = row[7]; /* 6th arg contains the reconnects value*/
    source->reconnects = (cur == NULL)? 0 : atoi(cur);
  } else {
    stream404_log ("The command specified in sql response doesn't exist %s try again with next entry", cur);
    mysql_free_result(res);  
    return playlist_sql_builtin_get_next(arg);
    //return NULL; 
  }

  /* okay parse the fields and check at the second one which contains the filename and then location */
  out = strdup(row[2]);

  if ((source->location == LOCAL) && (stream404_util_verify_file(out) == 0)) {
    stream404_log ("The file specified in sql response doesn't exist %s", out);
    stream404_util_free(out);
    mysql_free_result(res);
    /* this is not because the current file doesn't exists that we have to stop
       so call again this function */
    return playlist_sql_builtin_get_next(arg);
    //return NULL;
  }

  mysql_free_result(res);
  return out;
}

static char *
playlist_sql_builtin_get_next_urg (void *arg) 
{
  char *sql_text = "SELECT `S_ID`, `S_Code`, `S_Argument1`, `S_Argument2`, `S_Argument3`, `S_Argument4`, `S_Argument5`, `S_Argument6` FROM %s order by S_ID LIMIT 1";
  char *sql_text_del = "DELETE FROM %s WHERE S_ID = %s";
  //char *sql_currentstreamtable = "t_currentstream";
  char *sql_text_del_current = "DELETE FROM %s";
  char *sql_text_current = "INSERT into %s SELECT `S_Argument4`, now(), `S_UserID`, `S_Argument8`, `S_TrackID`, `S_DataID`, `S_Argument7`, `S_Argument1` FROM %s where S_ID = %s";
  //char *sql_streamlogtable = "t_streamlog";
  char *sql_text_stream_log = "INSERT into %s (S_Type,D_Datetime,S_DataID,S_FileName,S_Description,S_PlayTime) SELECT `S_Argument4`, now(), `S_DataID`, `S_Argument1`, `S_Argument6`, `S_Argument7` FROM %s where S_ID = %s";
  char c[4096];
  char *out, *cur;
  input_stream_t *source = arg;

  /* set these values to 0 */
  source->type = 0;
  source->spec = 0;

  /*
  if (!mysql) {
    * *hem* sql connect has been droped *
    if (!playlist_sql_builtin_open_playlist (sqlpm)) {
      stream404_log ("Could not connect to sql server.");
      stream404_setup_shutdown ();
      return NULL;
    }
  }
  */
  
 
  snprintf(c, strlen(sql_text) + strlen(stream404_config.sql.urg_table), sql_text, stream404_config.sql.urg_table); 

  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    return NULL;
  }

  res = mysql_store_result(&mysql);
  
  if (mysql_num_rows(res) == 0) {
    /* there is no next file to be played !! */
    stream404_log_error ("OMG ! No next file to play !!");
    mysql_free_result(res);
    return NULL;
  }

  row = mysql_fetch_row(res);

 

   /* purge current stream table*/
  snprintf(c, strlen(sql_text_del_current) + strlen(stream404_config.sql.currentstream_table), sql_text_del_current, stream404_config.sql.currentstream_table);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query2: %s\n%s", mysql_error(&mysql), c);
    mysql_free_result(res);
    return NULL;
  }
  /* then update current stream table with the current song played */
  snprintf(c, strlen(sql_text_current) + strlen(stream404_config.sql.currentstream_table) + strlen(stream404_config.sql.urg_table) + strlen(row[0]), sql_text_current, stream404_config.sql.currentstream_table, stream404_config.sql.urg_table, row[0]);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    stream404_log ("c = %s", c);  
    mysql_free_result(res);
    return NULL;
  }

  /* log current stream with the current song played */
  snprintf(c, strlen(sql_text_stream_log) + strlen(stream404_config.sql.streamlog_table) + strlen(stream404_config.sql.table) + strlen(row[0]), sql_text_stream_log, stream404_config.sql.streamlog_table, stream404_config.sql.table, row[0]);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error log stream sql query: %s", mysql_error(&mysql));
    mysql_free_result(res);
    return NULL;
  }

   /* delete this record */
  snprintf(c, strlen(sql_text_del) + strlen(stream404_config.sql.urg_table) + strlen(row[0]), sql_text_del, stream404_config.sql.urg_table, row[0]);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    mysql_free_result(res);
    return NULL;
  }

    /* check for the second argument */
  cur = row[1];
  
  if (cur && (strncmp(cur, "PLAY", 4) == 0)) {
    source->spec = _FILE;
    /* get the file location */
    cur = row[2];
    if (cur && (strncasecmp(cur, "http://", 7) == 0)) {
      source->location = REMOTE;
    } else {
      source->location = LOCAL;
    }
    cur = row[3];
    if (cur && (strncasecmp(cur, "MP3", 3) == 0)) {
      source->type = STREAM404_INPUT_MP3;
    } 
    if (cur && (strncasecmp(cur, "OGG", 3) == 0)) {
      source->type = STREAM404_INPUT_VORBIS;
    } 
    if (cur && (strncasecmp(cur, "MP4", 3) == 0)) {
      source->type = STREAM404_INPUT_MP4;
    }
    if (cur && (strncasecmp(cur, "FLAC", 4) == 0)) {
      source->type = STREAM404_INPUT_MP4;
    }
    else {
      /* should stay as STREAM404_INPUT_UNKNOWN */
    }
  } else if (cur && (strncmp(cur, "LIVE", 4) == 0)) {
    source->location = REMOTE;
    source->spec = STREAM;
    cur = row[3];
    if (cur && (strncasecmp(cur, "MP3", 3) == 0)) {
      source->type = STREAM404_INPUT_MP3;
    } 
    if (cur && (strncasecmp(cur, "OGG", 3) == 0)) {
      source->type = STREAM404_INPUT_VORBIS;
    } 
    if (cur && (strncasecmp(cur, "MP4", 3) == 0)) {
      source->type = STREAM404_INPUT_MP4;
    }
    if (cur && (strncasecmp(cur, "FLAC", 4) == 0)) {
      source->type = STREAM404_INPUT_MP4;
    }
    else {
      /* should stay as STREAM404_INPUT_UNKNOWN */
    }
    cur = row[7]; /* 6th arg contains the reconnects value*/
    source->reconnects = (cur == NULL)? 0 : atoi(cur);
  } else {
    stream404_log ("The command specified in sql response doesn't exist %s try again with next entry", cur);
    mysql_free_result(res);  
    return playlist_sql_builtin_get_next(arg);
    //return NULL; 
  }

  /* okay parse the fields and check at the second one which contains the filename and then location */
  out = strdup(row[2]);

  if ((source->location == LOCAL) && stream404_util_verify_file(out) == 0) {
    stream404_log ("The file specified in sql response doesn't exist");
    stream404_util_free(out);
    mysql_free_result(res);
    /* this is not because the current file doesn't exists that we have to stop
       so call again this function. The difference is we look directly at
       streamstack and not pilotstack
    */
    return playlist_sql_builtin_get_next(arg);
    //return NULL;
  }

  mysql_free_result(res);
  return out;
}


/* return the new metadata to set */
static char* playlist_sql_get_metadata(void)
{
  char *metadata = NULL;
  char *sql_text = "SELECT `S_Data` FROM %s LIMIT 1";
  char *sql_text_del = "TRUNCATE table %s";
  char c[4096];
  
  snprintf(c, strlen(sql_text) + strlen(stream404_config.sql.metadata_table), sql_text, stream404_config.sql.metadata_table); 
  
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    return NULL;
  }

  res = mysql_store_result(&mysql);
  
  if (mysql_num_rows(res) == 0) {
    /* there is no next file to be played !! */
    stream404_log_error ("METADATA UPDATE : none !");
    mysql_free_result(res);
    return NULL;
  }

  row = mysql_fetch_row(res);
  
  metadata = strdup(row[0]);
  
  /* now delete the entry */
  snprintf(c, strlen(sql_text_del) + strlen(stream404_config.sql.metadata_table) + strlen(row[0]), sql_text_del, stream404_config.sql.metadata_table, row[0]);
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    mysql_free_result(res);
  }
  
  return metadata;
}


/* Return the current playlist file line number */
static int
playlist_sql_builtin_get_lineno (void)
{
  return 0;
}

/* Shutdown the builtin playlist handler */
static void
playlist_sql_builtin_shutdown (void)
{
  if (fp)
    stream404_util_fclose (fp);
  mysql_close(&mysql);
}
	



/* Verify that the user specified playlist actually exists */
static int
playlist_sql_builtin_open_playlist (playlist_module_t* pm)
{
  
  /* first of all test all parameters */
  if ((!stream404_config.sql.usocket) && (!stream404_config.sql.hostname)) {
    stream404_log ("No host nor unix socket set to connect to sql serv !");
    return 0;
  }
  if (stream404_config.sql.port == 0) {
    stream404_log ("No port set to connect to sql serv !");
    return 0;
  }
  if (!stream404_config.sql.sqltype) {
    stream404_log ("No sql server defined !");
    return 0;
  }
  if ((!stream404_config.sql.user) || (!stream404_config.sql.password)) {
    stream404_log ("No user nor password set to connect to sql server !");
    return 0;
  }
  if ((!stream404_config.sql.table) || (!stream404_config.sql.database)) {
    stream404_log ("No database nor table set to connect to sql server !");
    return 0;
  }
  

  /* initialize mysql struct */
  mysql_init(&mysql);
  
  if (!stream404_config.sql.usocket) {
    /* connect using host parameter */
    if (!mysql_real_connect(&mysql, stream404_config.sql.hostname, 
			    stream404_config.sql.user, stream404_config.sql.password, 
			    stream404_config.sql.database, stream404_config.sql.port, 
			    NULL,0)) {
      stream404_log ("Cannot connect to sql database !");
      return 0;
    }
  } else {
    /* connect using socket parameter */
    if (!mysql_real_connect(&mysql, NULL, stream404_config.sql.user, 
			    stream404_config.sql.password, stream404_config.sql.database,
			    stream404_config.sql.port, stream404_config.sql.usocket, 
			    0)) {
      stream404_log ("Cannot connect to sql database !");
      return 0;
    }
  }

  return 1;
}

/* XXXX kept for future purpose ?
   static int
   playlist_sql_builtin_reopen_playlist (playlist_module_t* pm)
   {
   stream404_log_debug ("Reopening sql connection");
   
   mysql_close(&mysql);
   
   if (! playlist_sql_builtin_open_playlist (&stream404_config.pm))
   return 0;
   
   return 1;
   }
*/


/* New process used to look at the pilotstream is not intended to be real time
   so we can afford to look every 5 seconds */
#define SLEEP_TIME 5

void get_urg_entry()
{
  char *sql_urgtext = "SELECT `S_ID`, `S_Code`, `S_Argument1`, `S_Argument2`, `S_Argument3`, `S_Argument4`, `S_Argument5`, `S_Argument6` FROM %s order by S_ID LIMIT 1";
  char *sql_urgtext_del = "DELETE FROM %s WHERE S_ID = %s";
  char c[4096];

  int pid = getppid();
  
  snprintf(c, strlen(sql_urgtext) + strlen(stream404_config.sql.urg_table), sql_urgtext, stream404_config.sql.urg_table); 
  
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    return;
  }
  
  res = mysql_store_result(&mysql);
  
  if (mysql_num_rows(res) == 0) {
    /* there is no next file to be played !! */
    mysql_free_result(res);
    return;
  }
  
  row = mysql_fetch_row(res);
  
  /* compare row[1] (S_Code) with PLAY, PLAYTO, LIVE etc */
  if (strncmp(row[1], "PLAYTO", 6) == 0) {
    int sl = atoi(row[2]);
    /* delete current entry */
    /* delete this record */
    snprintf(c, strlen(sql_urgtext_del) + strlen(stream404_config.sql.urg_table), sql_urgtext_del, stream404_config.sql.urg_table, row[0]);
    if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
      stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
      mysql_free_result(res);
      return;
    }
    /* sleep Argument_1 time */
    sleep(sl);
    /* and ask to play the next file */
    kill(pid, SIGUSR1);
  } else if (strncmp(row[1], "PLAY", 4) == 0) {
    /* we need to play the next file specified in arg
       send a signale to parent process */
    kill(pid, SIGUSR2);
  } else if (strncmp(row[1], "LIVE", 4) == 0) {
    /* we need to play the next file specified in arg
       send a signale to parent process */
    kill(pid, SIGUSR2);
  } else if (strncmp(row[1], "STOP", 4) == 0) {
    /* delete current entry */
    snprintf(c, strlen(sql_urgtext_del) + strlen(stream404_config.sql.urg_table), sql_urgtext_del, stream404_config.sql.urg_table, row[0]);
    if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
      stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
      mysql_free_result(res);
      return;
    }
    /* stop play the current file which means we jump to the next one */
    kill(pid, SIGUSR1);
  } else {
   /* unknown command */
    stream404_log ("The command specified in sql response doesn't exist %s try again with next entry", row[1]);
    /* delete current entry */
    snprintf(c, strlen(sql_urgtext_del) + strlen(stream404_config.sql.urg_table), sql_urgtext_del, stream404_config.sql.urg_table, row[0]);
  }
    
  mysql_free_result(res);
  return;
}

void get_metadata_entry() {
  char *sql_metadatatext = "SELECT `S_Data` FROM %s LIMIT 1";
  char c[4096];

  int pid = getppid();

  snprintf(c, strlen(sql_metadatatext) + strlen(stream404_config.sql.metadata_table), sql_metadatatext, stream404_config.sql.metadata_table); 
  
  if (mysql_real_query(&mysql, c, strlen(c)) < 0) {
    stream404_log ("Error requesting sql query: %s", mysql_error(&mysql));
    return;
  }
  
  res = mysql_store_result(&mysql);
  
  if (mysql_num_rows(res) != 0) {
    /* there is a metadata !! */
    kill(pid, SIGPOLL);
  }
  mysql_free_result(res);

  return;
}

void launch_loop(void)
{

  switch (stream404_config.childloop = fork()) {
  case -1:
    /* cannot create process */
    stream404_log ("Cannot create process");
    stream404_setup_shutdown ();
    break;
  case 0:
    /* client part */
    break;
  default:
    /* parent return*/
    return;
  }

  setproctitle("stream404", "child loop");
  
  /* initialize mysql struct */
  mysql_init(&mysql);

  if (!stream404_config.sql.usocket) {
    /* connect using host parameter */
    if (!mysql_real_connect(&mysql, stream404_config.sql.hostname, 
			    stream404_config.sql.user, stream404_config.sql.password, 
			    stream404_config.sql.database, stream404_config.sql.port, 
			    NULL,0)) {
      stream404_log ("Cannot connect to sql database !");
      exit(0);
    }
  } else {
    /* connect using socket parameter */
    if (!mysql_real_connect(&mysql, NULL, stream404_config.sql.user, 
			    stream404_config.sql.password, stream404_config.sql.database,
			    stream404_config.sql.port, stream404_config.sql.usocket, 
			    0)) {
      stream404_log ("Cannot connect to sql database !");
      exit(0);
    }
  } 

  while(1) {
    sleep(SLEEP_TIME);

    if (getppid() <= 1) {
      /* our parent died so does it */
      /* 1 because it is the init process */
      exit(0);
    }

    get_urg_entry();
    get_metadata_entry();

  }
  
}

